﻿
using System;
using System.Collections.Generic;
using CppDepend.Helpers;
using CppDepend.Path;
using CppDepend.PowerTools.Base;
using CppDepend.PowerTools.SharedUtils;
using CppDepend.Project;
using CppDepend.DotNet.VisualStudio;
using NDepend;
namespace CppDepend.PowerTools.AnalyzeCodeOnDisk {
   class AnalyzeCodeOnDiskPowerTool : IPowerTool {
      public string Name { get { return "Analyze Code on my Disk"; } }

      public string[] Description {
         get {
            return new [] {
            "Gather Visual Studio solutions and projects from Visual Studio",
            "Most Recently Used (MRU) list, for each Visual Studio installed version.",
            "Gather all Debug projects from these solutions and analyze them.",
         };
         }
      }

      public void Run() {
         var projectManager = new NDependServicesProvider().ProjectManager;
         var visualStudioManager = new NDependServicesProvider().VisualStudioManager;

         //
         // Gather all Most Recent Used VS solutions and projects file path referenced in registry 
         //
         var vsFilesPath = new List<IAbsoluteFilePath>();
         foreach (var vsVersion in new[] { VisualStudioVersion.V7_2003, VisualStudioVersion.V8_2005, VisualStudioVersion.V9_2008, VisualStudioVersion.V10_2010 }) {
            if (!visualStudioManager.IsVisualStudioVersionIntalled(vsVersion)) { continue; }
            vsFilesPath.AddRange(visualStudioManager.GetMostRecentlyUsedVisualStudioSolutionOrProject(vsVersion));
         }
         Console.WriteLine(vsFilesPath.Count + " VS sln or proj files found");

         //
         // Gather all Debug projects built from these VS solutions and projects
         //
         var projectsPath = new List<IAbsoluteFilePath>();
         foreach (var vsFilePath in vsFilesPath) {
            projectsPath.Add(vsFilePath);
         }

         //
         // Create temporary project
         //
         var project = projectManager.CreateTemporaryProject(projectsPath, TemporaryProjectMode.Temporary);

         
      

         //
         // Run analysis
         //
         var analysisResult = ProjectAnalysisUtils.RunAnalysisShowProgressOnConsole(project);

         //
         // Show results
         //
         ProjectAnalysisUtils.ShowConsoleAnalysisResult(analysisResult.CodeBase);


         Console.WriteLine("The analysis result produced can now be consummed from the CppDepend project \"" + project.Properties.Name + "\"");
      }
   }
}
