﻿

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Windows.Forms;
using CppDepend.DotNet;
using CppDepend.Path;
using CppDepend.PowerTools.Base;
using CppDepend.PowerTools.SharedUtils;
using CppDepend.Project;
using NDepend;
namespace CppDepend.PowerTools.AnalyzeProjectsInFolder {

   class AnalyzeProjectsInFolder : IPowerTool {
      public string Name { get { return "Analyze Projects in Folder"; } }

      public string[] Description {
         get {
            return new[] {
            "Gather C/C++ projects under the folder specified by the user and analyze them.",
            "User can choose to do a recursive search in the folder."
         };
         }
      }

      public void Run() {

         //
         // Get dir
         //

       
         var folderBrowserDialog = new FolderBrowserDialog {ShowNewFolderButton = false};
         if (folderBrowserDialog.ShowDialog() != DialogResult.OK) { return; }
         var dir = folderBrowserDialog.SelectedPath.ToAbsoluteDirectoryPath();

         //
         // Get recursive
         //
         bool recursive = false;
         Console.BackgroundColor = ConsoleColor.Black;
         Console.ForegroundColor = ConsoleColor.Yellow;
         Console.WriteLine("Search projects recursive under {" + dir.ToString() + "} ?");
         Console.WriteLine("Yes?  y    or No? (another key)");
         var consoleKeyInfo = Console.ReadKey();
         if (consoleKeyInfo.KeyChar == 'y') {
            recursive = true;
         }
         Console.ForegroundColor = ConsoleColor.White;


         //
         // Get projectsPath
         //
         var dotNetManager = new NDependServicesProvider().DotNetManager;
         var projectsPath = new List<IAbsoluteFilePath>();
         var cursorTop = Console.CursorTop;
         if (!recursive) {
            ScanDir(dir, projectsPath, dotNetManager, cursorTop);
         } else {
            ScanDirRecursive(dir, projectsPath, dotNetManager, Console.CursorTop);
         }
         Console.CursorTop = cursorTop;
         Console.CursorLeft = 0;
         ConsoleUtils.ShowNLinesOnConsole(10, ConsoleColor.Black);
         Console.CursorTop = cursorTop;
         Console.CursorLeft = 0;

         //
         // Get project
         //
         Console.ForegroundColor = ConsoleColor.Yellow;
         Console.WriteLine(projectsPath.Count + " projects found");
         Console.WriteLine(projectsPath.Select(path => path.FileNameWithoutExtension).Distinct(StringComparer.InvariantCultureIgnoreCase).Count() + " projects with distint names found");
         Console.WriteLine("Create the CppDepend temporary project.");
         var projectManager = new NDependServicesProvider().ProjectManager;
         var project = projectManager.CreateTemporaryProject(projectsPath, TemporaryProjectMode.Temporary);

         //
         // Run analysis
         //
         Console.ForegroundColor = ConsoleColor.Yellow;
         Console.WriteLine("Run Analysis!");
         var analysisResult = ProjectAnalysisUtils.RunAnalysisShowProgressOnConsole(project);

         //
         // Show results
         //
         ProjectAnalysisUtils.ShowConsoleAnalysisResult(analysisResult.CodeBase);
      }

      private static void ScanDirRecursive(IAbsoluteDirectoryPath dir, List<IAbsoluteFilePath> projectsPath, IDotNetManager dotNetManager, int cursorTop) {
         Debug.Assert(dir != null);
         Debug.Assert(dir.Exists);
         Debug.Assert(projectsPath != null);
         Debug.Assert(dotNetManager != null);
         ScanDir(dir, projectsPath, dotNetManager, cursorTop);
         foreach (var dirChild in dir.ChildrenDirectoriesPath) {
            ScanDirRecursive(dirChild, projectsPath, dotNetManager,cursorTop);
         }
      }


      private static void ScanDir(IAbsoluteDirectoryPath dir, List<IAbsoluteFilePath> projectsPath, IDotNetManager dotNetManager, int cursorTop) {
         Debug.Assert(dir != null);
         Debug.Assert(dir.Exists);
         Debug.Assert(projectsPath != null);
         Debug.Assert(dotNetManager != null);

         Console.CursorTop = cursorTop;
         Console.CursorLeft = 0;
         Console.WriteLine(projectsPath.Count + " projects found");
         Console.Write("Scanning {" + dir.ToString() + "} ");
         foreach (var filePath in dir.ChildrenFilesPath) {
         if (filePath.FileExtension == ".vcproj" || filePath.FileExtension == ".vcproj" || filePath.FileExtension == ".vcxproj" || filePath.FileExtension == ".sln" || filePath.FileExtension == ".build" || filePath.FileExtension == ".sl" || filePath.FileExtension == ".proj")
            {
                projectsPath.Add(filePath);
            }
         }
         ConsoleUtils.ShowNLinesOnConsole(Console.CursorTop - cursorTop +1, ConsoleColor.Black);
      }

   }
}